@@ -36,12 +36,15 @@ static struct nfs_mount_data mount_data = {
 };
 
 int nfs_port;
+int nfs_version;
 
 static struct int_opts {
 	char *name;
 	int *val;
 } int_opts[] = {
 	{"port",	&nfs_port},
+	{"nfsvers",	&nfs_version},
+	{"vers",	&nfs_version},
 	{"rsize",	&mount_data.rsize},
 	{"wsize",	&mount_data.wsize},
 	{"timeo",	&mount_data.timeo},
@@ -102,7 +105,8 @@ static void parse_opts(char *opts)
 	while ((cp = strsep(&opts, ",")) != NULL) {
 		if (*cp == '\0')
 			continue;
-		if ((val = strchr(cp, '=')) != NULL) {
+		val = strchr(cp, '=');
+		if (val != NULL) {
 			struct int_opts *opts = int_opts;
 			*val++ = '\0';
 			while (opts->name && strcmp(opts->name, cp) != 0)
@@ -128,6 +132,22 @@ static void parse_opts(char *opts)
 			}
 		}
 	}
+	/* If new-style options "nfsvers=" or "vers=" are passed, override
+	   old "v2" and "v3" options */
+	if (nfs_version != 0) {
+		switch (nfs_version) {
+		case 2:
+			mount_data.flags &= ~NFS_MOUNT_VER3;
+			break;
+		case 3:
+			mount_data.flags |= NFS_MOUNT_VER3;
+			break;
+		default:
+			fprintf(stderr, "%s: bad NFS version '%d'\n",
+				progname, nfs_version);
+			longjmp(abort_buf, 1);
+		}
+	}
 }
 
 static uint32_t parse_addr(const char *ip)
@@ -167,7 +187,7 @@ int nfsmount_main(int argc, char *argv[])
 	int c;
 	const char *portmap_file;
 	pid_t spoof_portmap;
-	int err;
+	int err, ret;
 
 	if ((err = setjmp(abort_buf)))
 		return err;
@@ -207,13 +227,16 @@ int nfsmount_main(int argc, char *argv[])
 
 	hostname = rem_path = argv[optind];
 
-	if ((rem_name = strdup(rem_path)) == NULL) {
+	rem_name = strdup(rem_path);
+	if (rem_name == NULL) {
 		perror("strdup");
 		return 1;
 	}
 
-	if ((rem_path = strchr(rem_path, ':')) == NULL) {
+	rem_path = strchr(rem_path, ':');
+	if (rem_path == NULL) {
 		fprintf(stderr, "%s: need a server\n", progname);
+		free(rem_name);
 		return 1;
 	}
 
@@ -221,6 +244,7 @@ int nfsmount_main(int argc, char *argv[])
 
 	if (*rem_path != '/') {
 		fprintf(stderr, "%s: need a path\n", progname);
+		free(rem_name);
 		return 1;
 	}
 
@@ -233,29 +257,32 @@ int nfsmount_main(int argc, char *argv[])
 
 	check_path(path);
 
-#if! _KLIBC_NO_MMU
+#if !_KLIBC_NO_MMU
 	/* Note: uClinux can't fork(), so the spoof portmapper is not
 	   available on uClinux. */
 	if (portmap_file)
 		spoof_portmap = start_dummy_portmap(portmap_file);
 
-	if (spoof_portmap == -1)
+	if (spoof_portmap == -1) {
+		free(rem_name);
 		return 1;
+	}
 #endif
 
+	ret = 0;
 	if (nfs_mount(rem_name, hostname, server, rem_path, path,
 		      &mount_data) != 0)
-		return 1;
+		ret = 1;
 
 	/* If we set up the spoofer, tear it down now */
 	if (spoof_portmap) {
-		kill(SIGTERM, spoof_portmap);
-		while (waitpid(spoof_portmap, NULL, 0) == -1 &&
-		       errno == EINTR)
-		  ;
+		kill(spoof_portmap, SIGTERM);
+		while (waitpid(spoof_portmap, NULL, 0) == -1
+		       && errno == EINTR)
+			;
 	}
 
 	free(rem_name);
 
-	return 0;
+	return ret;
 }
